import os

from e3.fs import cp
from e3.os.fs import df
from e3.testsuite.driver.classic import ClassicTestDriver
from e3.testsuite.control import YAMLTestControlCreator

from drivers import gprbuild, run_test_program


class BasicTestDriver(ClassicTestDriver):
    """Default GNATcoll testsuite driver.

    In order to declare a test:

    1- Create a directory with a test.yaml inside
    2- Add test sources in that directory
    3- Add a main called test.adb that use support/test_assert.ads package.
    4- Do not put test.gpr there, it breaks the test, if you need a project
       file for testing, name it something else.
    5- If you need additional files for you test, list them in test.yaml:
       data:
           - "your_file1"
           - "your_file2"
    """

    # We want to copy only specific files (files referenced by the "data" key
    # in test.yaml).
    copy_test_directory = False

    @property
    def test_control_creator(self):
        return YAMLTestControlCreator({
            'env': self.env,
            'test_env': self.test_env,
            'disk_space': lambda: df(self.env.working_dir)
        })

    def run(self):
        # Build the test program
        if self.test_env.get('no-coverage'):
            gpr_project_path = self.env.gnatcoll_debug_gpr_dir
        else:
            gpr_project_path = self.env.gnatcoll_gpr_dir
        gprbuild(self, gcov=self.env.gcov, gpr_project_path=gpr_project_path)

        # Copy the requested data files
        for data in self.test_env.get('data', []):
            cp(os.path.join(self.test_env['test_dir'], data),
               self.test_env['working_dir'], recursive=True)

        # Run the test program
        test_exe = self.test_env.get('test_exe', 'obj/test')
        process = run_test_program(
            self,
            [os.path.join(self.test_env['working_dir'], test_exe)],
            timeout=self.default_process_timeout)
        self.output += process.out.decode('utf-8')

    def compute_failures(self):
        return (['Success marker not found']
                if '<=== TEST PASSED ===>' not in self.result.log.log else [])



import logging
import os

from e3.os.process import Run

from drivers import bin_check_call


COVERAGE_LEVEL = 'stmt+decision'


def list_to_file(str_list, filename):
    """Write a list of strings to a text file.

    :param list[str] str_list: List of strings to write.
    :param str filename: Name of the destination file.
    """
    with open(filename, 'w') as f:
        for item in str_list:
            f.write(item + '\n')


def gnatcov_run(driver, cmd, test_name=None, result=None, **kwargs):
    """
    Wrapper for `bin_check_call` that runs the process under
    "gnatcov run" and that produces a checkpoint in
    `driver.env.checkpoints_dir` for the corresponding partial coverage report.
    """
    test_name = test_name or driver.test_env['test_name']
    trace_file = os.path.join(driver.test_env['working_dir'],
                              '{}.trace'.format(test_name))
    checkpoint_file = os.path.join(driver.env.checkpoints_dir,
                                   '{}.ckpt'.format(test_name))

    cmd = ['gnatcov', 'run', '-o', trace_file, '-eargs'] + cmd
    result = bin_check_call(driver, cmd, test_name, result, **kwargs)

    p = Run(['gnatcov', 'coverage',
             '--level={}'.format(COVERAGE_LEVEL),
             '--scos=@{}'.format(driver.env.ali_files_list),
             '--save-checkpoint={}'.format(checkpoint_file),
             trace_file])
    if p.status:
        logging.error('converting gnatcov trace file to checkpoint failed:\n'
                      '{}'.format(p.out))

    return result


def produce_report(output_dir, checkpoint_list, src_dir):
    """Produce a coverage reports.

    :param str output_dir: Name of the directory to contain the DHTML coverage
        report.
    :param str checkpoint_list: Name of the file that contains the list of
        checkpoints to use.
    :param str src_dir: Name of the directory that contains installed sources.
    """
    args = ['gnatcov', 'coverage', '--annotate=dhtml',
            '--level={}'.format(COVERAGE_LEVEL),
            '--output-dir={}'.format(output_dir),
            '--checkpoint=@{}'.format(checkpoint_list),

            # TODO: GNATcoverage is not be able to find the source file for a
            # unit that is not used by any test program. This is a problem for
            # units that are not tested at all. Let it know where to find the
            # source file to avoid spurious warnings. Note that these units are
            # reported as uncovered in any case.
            '--source-search={}'.format(src_dir)]
    p = Run(args, output=None)
    if p.status:
        logging.error('could not produce the coverage report:\n'
                      '{}'.format(p.out))
    elif p.out:
        logging.info('output of "gnatcov coverage" is not empty:\n'
                     '{}'.format(p.out))



from e3.fs import rm
from e3.testsuite.result import TestStatus, TestResult
from drivers import GNATcollTestDriver, gprbuild
import os


class DataValidationDriver(GNATcollTestDriver):
    """Data validation driver.

    For each test program call the program with data file defined in
    data_files key of the test. If the program returns 0 assume that
    the test passed.
    """

    def add_test(self, dag):
        self.add_fragment(dag, 'build')

        tear_down_deps = []
        for data_file, description in self.test_env['data_files'].items():
            tear_down_deps.append(data_file)
            self.add_fragment(
                dag,
                data_file,
                fun=self.run_subtest_for(data_file, description),
                after=['build'])
        self.add_fragment(dag, 'tear_down', after=tear_down_deps)

    def run_subtest_for(self, data_file, description):

        def run_subtest(previous_values, slot):
            test_name = self.test_name + '.' + data_file
            result = TestResult(test_name, env=self.test_env)

            if not previous_values['build']:
                return TestStatus.FAIL

            process = self.run_test_program(
                [os.path.join(self.test_env['working_dir'],
                              self.test_env.get('validator', 'obj/test')),
                 os.path.join(self.test_env['test_dir'], data_file)],
                test_name=test_name,
                result=result,
                timeout=self.process_timeout)

            return self.validate_result(process, data_file, result)

        return run_subtest

    def validate_result(self, process, data_file, result):
        # Test passes as soon as there is this magic string is its output. Note
        # that we push a test result only on failure.
        if b'<=== TEST PASSED ===>' not in process.out:
            result.set_status(TestStatus.FAIL)
            self.push_result(result)
        return True

    def tear_down(self, previous_values, slot):
        # If the test program build failed, there is nothing we can do (and the
        # result for this test is alredy pushed anyway).
        if not previous_values.get('build'):
            return False

        failures = [v for v in previous_values.values() if
                    not isinstance(v, TestStatus) or v != TestStatus.PASS]
        if failures:
            self.result.set_status(TestStatus.FAIL,
                                   msg="%s subtests failed" % len(failures))
        else:
            self.result.set_status(TestStatus.PASS)

        self.push_result()

        if self.env.enable_cleanup:
            rm(self.test_env['working_dir'], recursive=True)

    def build(self, previous_values, slot):
        return gprbuild(self, gcov=self.env.gcov,
                        gpr_project_path=self.env.gnatcoll_gpr_dir)



import logging
import os
import subprocess

from e3.fs import mkdir
from e3.os.process import Run, get_rlimit
from e3.testsuite import TestAbort
from e3.testsuite.driver import TestDriver
from e3.testsuite.driver.classic import (
    ClassicTestDriver, ProcessResult, TestAbortWithFailure
)
from e3.testsuite.process import check_call
from e3.testsuite.result import Log, TestStatus


# Root directory of respectively the testsuite and the gnatcoll
# repository.
TESTSUITE_ROOT_DIR = os.path.dirname(
    os.path.dirname(os.path.abspath(__file__)))
GNATCOLL_ROOT_DIR = os.path.dirname(TESTSUITE_ROOT_DIR)

DEFAULT_TIMEOUT = 5 * 60  # 5 minutes


def make_gnatcoll(work_dir, debug=False, gcov=False, gnatcov=False):
    """Build gnatcoll core with or without gcov instrumentation.

    :param str work_dir: Working directory. GNATcoll is built in `build` subdir
        and installed in `install` subdir.

    :param bool debug: Whether to build GNATCOLL in debug mode. Otherwise, use
        the prod mode. Note that gcov and gnatcov modes automatically enable
        debug mode.

    :param bool gcov: If true, build GNATCOLL with gcov instrumentation in
        debgu mode.

    :param bool gnatcov: If True, build GNATCOLL with the compile options that
        GNATcoverage require in debug mode.

    :return: A triplet (project path, source path, object path).
    :rtype: (str, str, str)
    :raise AssertError: In case compilation of installation fails.
    """
    assert not (gcov and gnatcov)

    if gcov:
        tag = ' (gcov)'
    elif gnatcov:
        tag = ' (gnatcov)'
    else:
        tag = ''
    logging.info('Compiling gnatcoll{}'.format(tag))

    # Create build tree structure
    build_dir = os.path.join(work_dir, 'build')
    install_dir = os.path.join(work_dir, 'install')
    mkdir(build_dir)
    mkdir(install_dir)

    # Compute make invocation
    make_gnatcoll_cmd = [
        'make', '-f', os.path.join(GNATCOLL_ROOT_DIR, 'Makefile'),
        'ENABLE_SHARED=no',
        'BUILD={}'.format('DEBUG' if debug or gcov or gnatcov else 'PROD')]
    if gcov:
        make_gnatcoll_cmd += [
            'GPRBUILD_OPTIONS=-cargs -fprofile-arcs -ftest-coverage'
            ' -cargs:Ada -gnatwn'
            ' -gargs']
    elif gnatcov:
        make_gnatcoll_cmd += [
            'GPRBUILD_OPTIONS=-cargs -fdump-scos -fpreserve-control-flow'
            ' -gargs']

    # Build & Install
    p = Run(make_gnatcoll_cmd, cwd=build_dir, timeout=DEFAULT_TIMEOUT)
    assert p.status == 0, "gnatcoll build failed:\n%s" % p.out

    p = Run(make_gnatcoll_cmd + ['prefix=%s' % install_dir, 'install'],
            cwd=build_dir, timeout=DEFAULT_TIMEOUT)
    assert p.status == 0, "gnatcoll installation failed:\n%s" % p.out

    return (os.path.join(install_dir, 'share', 'gpr'),
            os.path.join(install_dir, 'include', 'gnatcoll'),
            os.path.join(build_dir, 'obj', 'gnatcoll', 'static'))


def gprbuild(driver,
             project_file=None,
             cwd=None,
             gcov=False,
             scenario=None,
             gpr_project_path=None,
             timeout=DEFAULT_TIMEOUT,
             **kwargs):
    """Launch gprbuild.

    :param project_file: project file to compile. If None, we looks first for
        a test.gpr in the test dir and otherwise fallback on the common
        test.gpr project of the support subdir of the testsuite.
    :type project_file: str
    :param cwd: directory in which to run gprbuild. If None the gprbuild build
        is run in the default working dir for the test.
    :type cwd: str | None
    :param gcov: if True link with gcov libraries
    :type gcov: bool
    :param scenario: scenario variable values
    :type scenario: dict
    :param gpr_project_path: if not None prepent this value to GPR_PROJECT_PATH
    :type gpr_project_path: None | str
    :param kwargs: additional keyword arguements are passed to
        e3.testsuite.process.check_call function
    :return: True on successful completion
    :rtype: bool
    """
    if scenario is None:
        scenario = {}

    if project_file is None:
        project_file = os.path.join(driver.test_env['test_dir'],
                                    'test.gpr')
        if not os.path.isfile(project_file):
            project_file = os.path.join(TESTSUITE_ROOT_DIR,
                                        'support', 'test.gpr')
            scenario['TEST_SOURCES'] = driver.test_env['test_dir']

    if cwd is None:
        cwd = driver.test_env['working_dir']
    mkdir(cwd)
    gprbuild_cmd = [
        'gprbuild', '--relocate-build-tree', '-p', '-P', project_file]
    for k, v in scenario.items():
        gprbuild_cmd.append('-X%s=%s' % (k, v))
    if gcov:
        gprbuild_cmd += ['-largs', '-lgcov', '-cargs',
                         '-fprofile-arcs', '-ftest-coverage', '-g']
    elif driver.env.gnatcov:
        # TODO: GNATcoverage relies on debug info to do its magic. It needs
        # consistent paths to source files in the debug info, so do not build
        # tests with debug info, as they will reference installed sources
        # (while GNATCOLL objects reference original sources).
        gprbuild_cmd += ['-g0']

    # Adjust process environment
    env = kwargs.pop('env', None)
    ignore_environ = kwargs.pop('ignore_environ', True)
    if env is None:
        env = {}
        ignore_environ = False
    if gpr_project_path:
        new_gpr_path = gpr_project_path
        if 'GPR_PROJECT_PATH' in os.environ:
            new_gpr_path += os.path.pathsep + os.environ['GPR_PROJECT_PATH']
        env['GPR_PROJECT_PATH'] = new_gpr_path

    check_call(
        driver,
        gprbuild_cmd,
        cwd=cwd,
        env=env,
        ignore_environ=ignore_environ,
        timeout=timeout,
        **kwargs)
    # If we get there it means the build succeeded.
    return True


def bin_check_call(driver, cmd, test_name=None, result=None, timeout=None,
               env=None, cwd=None):
    if cwd is None and "working_dir" in driver.test_env:
        cwd = driver.test_env["working_dir"]
    if result is None:
        result = driver.result
    if test_name is None:
        test_name = driver.test_name
    if timeout is not None:
        cmd = [get_rlimit(), str(timeout)] + cmd

    # Use directly subprocess instead of e3.os.process.Run, since the latter
    # does not handle binary outputs.
    subp = subprocess.Popen(
        cmd, cwd=cwd, env=env, stdin=subprocess.DEVNULL,
        stdout=subprocess.PIPE, stderr=subprocess.STDOUT
    )
    stdout, _ = subp.communicate()
    process = ProcessResult(subp.returncode, stdout)
    result.processes.append(
        {
            "output": Log(stdout),
            "status": process.status,
            "cmd": cmd,
            "timeout": timeout,
            "env": env,
            "cwd": cwd,
        }
    )

    # Append the status code and process output to the log to ease post-mortem
    # investigation.
    result.log += "Status code: {}\n".format(process.status)
    result.log += "Output:\n"
    try:
        stdout = stdout.decode('utf-8')
    except UnicodeDecodeError:
        stdout = str(stdout)
    result.log += stdout

    if process.status != 0:
        if isinstance(driver, ClassicTestDriver):
            raise TestAbortWithFailure('command call fails')
        else:
            result.set_status(TestStatus.FAIL, "command call fails")
            driver.push_result(result)
            raise TestAbort
    return process


def run_test_program(driver, cmd, test_name=None, result=None, **kwargs):
    """
    Run a test program. This dispatches to running it under Valgrind or
    "gnatcov run", depending on the testsuite options.
    """
    from drivers.gnatcov import gnatcov_run
    from drivers.valgrind import check_call_valgrind

    if driver.env.valgrind:
        wrapper = check_call_valgrind
    elif driver.env.gnatcov:
        wrapper = gnatcov_run
    else:
        wrapper = bin_check_call

    return wrapper(driver, cmd, test_name, result, **kwargs)


class GNATcollTestDriver(TestDriver):
    """Abstract class to share some common facilities."""

    DEFAULT_TIMEOUT = 5 * 60  # 5 minutes

    @property
    def process_timeout(self):
        """Timeout (in seconds) for subprocess to launch."""
        return self.test_env.get('timeout', self.DEFAULT_TIMEOUT)

    def run_test_program(self, cmd, test_name=None, result=None, **kwargs):
        return run_test_program(self, cmd, test_name, result, **kwargs)



XML = r'''<?xml version="1.0"?>
<GPS>
  <documentation_file>
     <shell>Editor.edit "gnatcoll-format_columns_vertical.ads"</shell>
     <descr></descr>
     <menu>/Help/GNAT Runtime/</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-format_columns_vertical_xstring.ads"</shell>
     <descr></descr>
     <menu>/Help/GNAT Runtime/</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-any_types.ads"</shell>
     <descr>GNATCOLL.Any_Types</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Any__Types/&lt;Any__Types&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-any_types-python.ads"</shell>
     <descr>GNATCOLL.Any_Types.Python</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Any__Types/Python</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-arg_lists.ads"</shell>
     <descr>GNATCOLL.Arg_Lists</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Arg__Lists</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-asserts.ads"</shell>
     <descr>GNATCOLL.Asserts</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Asserts</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-atomic.ads"</shell>
     <descr>GNATCOLL.Atomic</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Atomic</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-boyer_moore.ads"</shell>
     <descr>GNATCOLL.Boyer_Moore</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Boyer__Moore</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-config.ads"</shell>
     <descr>GNATCOLL.Config</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Config</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-email.ads"</shell>
     <descr>GNATCOLL.Email</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Email/&lt;Email&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-email-mailboxes.ads"</shell>
     <descr>GNATCOLL.Email.Mailboxes</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Email/Mailboxes</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-email-parser.ads"</shell>
     <descr>GNATCOLL.Email.Parser</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Email/Parser</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-email-utils.ads"</shell>
     <descr>GNATCOLL.Email.Utils</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Email/Utils</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-formatters.ads"</shell>
     <descr>GNATCOLL.Formatters</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Formatters</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-gmp.ads"</shell>
     <descr>GNATCOLL.GMP</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/GMP/&lt;GMP&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-gmp-integers.ads"</shell>
     <descr>GNATCOLL.GMP.Integers</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/GMP/Integers/&lt;Integers&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-gmp-integers-io.ads"</shell>
     <descr>GNATCOLL.GMP.Integers.IO</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/GMP/Integers/IO</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-gmp-integers-misc.ads"</shell>
     <descr>GNATCOLL.GMP.Integers.Misc</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/GMP/Integers/Misc</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-gmp-integers-number_theoretic.ads"</shell>
     <descr>GNATCOLL.GMP.Integers.Number_Theoretic</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/GMP/Integers/Number__Theoretic</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-gmp-integers-random.ads"</shell>
     <descr>GNATCOLL.GMP.Integers.Random</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/GMP/Integers/Random</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-gmp-integers-root_extraction.ads"</shell>
     <descr>GNATCOLL.GMP.Integers.Root_Extraction</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/GMP/Integers/Root__Extraction</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-gmp-lib.ads"</shell>
     <descr>GNATCOLL.GMP.Lib</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/GMP/Lib</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-gmp-random_state.ads"</shell>
     <descr>GNATCOLL.GMP.Random_State</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/GMP/Random__State</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-geometry.ads"</shell>
     <descr>GNATCOLL.Geometry</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Geometry</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-io-native.ads"</shell>
     <descr>GNATCOLL.IO.Native</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/IO/Native</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-io-remote.ads"</shell>
     <descr>GNATCOLL.IO.Remote</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/IO/Remote/&lt;Remote&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-io-remote-unix.ads"</shell>
     <descr>GNATCOLL.IO.Remote.Unix</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/IO/Remote/Unix</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-io-remote-windows.ads"</shell>
     <descr>GNATCOLL.IO.Remote.Windows</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/IO/Remote/Windows</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-iconv.ads"</shell>
     <descr>GNATCOLL.Iconv</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Iconv</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-json.ads"</shell>
     <descr>GNATCOLL.JSON</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/JSON</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-memory.ads"</shell>
     <descr>GNATCOLL.Memory</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Memory</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-mmap.ads"</shell>
     <descr>GNATCOLL.Mmap</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Mmap/&lt;Mmap&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-paragraph_filling.ads"</shell>
     <descr>GNATCOLL.Paragraph_Filling</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Paragraph__Filling</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-pools.ads"</shell>
     <descr>GNATCOLL.Pools</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Pools</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-projects.ads"</shell>
     <descr>GNATCOLL.Projects</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Projects/&lt;Projects&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-projects-aux.ads"</shell>
     <descr>GNATCOLL.Projects.Aux</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Projects/Aux</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-promises.ads"</shell>
     <descr>GNATCOLL.Promises</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Promises</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-python.ads"</shell>
     <descr>GNATCOLL.Python</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Python</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-ravenscar.ads"</shell>
     <descr>GNATCOLL.Ravenscar</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Ravenscar/&lt;Ravenscar&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-ravenscar-multiple_queue_cyclic_server.ads"</shell>
     <descr>GNATCOLL.Ravenscar.Multiple_Queue_Cyclic_Server</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Ravenscar/Multiple__Queue__Cyclic__Server</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-ravenscar-multiple_queue_sporadic_server.ads"</shell>
     <descr>GNATCOLL.Ravenscar.Multiple_Queue_Sporadic_Server</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Ravenscar/Multiple__Queue__Sporadic__Server</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-ravenscar-simple_cyclic_task.ads"</shell>
     <descr>GNATCOLL.Ravenscar.Simple_Cyclic_Task</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Ravenscar/Simple__Cyclic__Task</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-ravenscar-simple_sporadic_task.ads"</shell>
     <descr>GNATCOLL.Ravenscar.Simple_Sporadic_Task</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Ravenscar/Simple__Sporadic__Task</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-ravenscar-sporadic_server.ads"</shell>
     <descr>GNATCOLL.Ravenscar.Sporadic_Server</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Ravenscar/Sporadic__Server</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-ravenscar-sporadic_server_with_callback.ads"</shell>
     <descr>GNATCOLL.Ravenscar.Sporadic_Server_With_Callback</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Ravenscar/Sporadic__Server__With__Callback</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-ravenscar-timed_out_sporadic_server.ads"</shell>
     <descr>GNATCOLL.Ravenscar.Timed_Out_Sporadic_Server</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Ravenscar/Timed__Out__Sporadic__Server</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-ravenscar-timers.ads"</shell>
     <descr>GNATCOLL.Ravenscar.Timers</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Ravenscar/Timers/&lt;Timers&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-ravenscar-timers-one_shot_timer.ads"</shell>
     <descr>GNATCOLL.Ravenscar.Timers.One_Shot_Timer</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Ravenscar/Timers/One__Shot__Timer</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-ravenscar-utils.ads"</shell>
     <descr>GNATCOLL.Ravenscar.Utils</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Ravenscar/Utils</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-readline.ads"</shell>
     <descr>GNATCOLL.Readline</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Readline</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-refcount.ads"</shell>
     <descr>GNATCOLL.Refcount</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Refcount/&lt;Refcount&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-refcount-weakref.ads"</shell>
     <descr>GNATCOLL.Refcount.Weakref</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Refcount/Weakref</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-remote.ads"</shell>
     <descr>GNATCOLL.Remote</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Remote/&lt;Remote&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-remote-db.ads"</shell>
     <descr>GNATCOLL.Remote.Db</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Remote/Db</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql.ads"</shell>
     <descr>GNATCOLL.SQL</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/&lt;SQL&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql-exec.ads"</shell>
     <descr>GNATCOLL.SQL.Exec</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/Exec/&lt;Exec&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql-exec-tasking.ads"</shell>
     <descr>GNATCOLL.SQL.Exec.Tasking</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/Exec/Tasking</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql-inspect.ads"</shell>
     <descr>GNATCOLL.SQL.Inspect</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/Inspect</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql-orm.ads"</shell>
     <descr>GNATCOLL.SQL.Orm</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/Orm/&lt;Orm&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql-orm-impl.ads"</shell>
     <descr>GNATCOLL.SQL.Orm.Impl</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/Orm/Impl</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql-postgres.ads"</shell>
     <descr>GNATCOLL.SQL.Postgres</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/Postgres/&lt;Postgres&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql-postgres-gnade.ads"</shell>
     <descr>GNATCOLL.SQL.Postgres.Gnade</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/Postgres/Gnade</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql-ranges.ads"</shell>
     <descr>GNATCOLL.SQL.Ranges</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/Ranges</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql-sessions.ads"</shell>
     <descr>GNATCOLL.SQL.Sessions</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/Sessions</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql-sqlite.ads"</shell>
     <descr>GNATCOLL.SQL.Sqlite</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/Sqlite/&lt;Sqlite&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql-sqlite-gnade.ads"</shell>
     <descr>GNATCOLL.SQL.Sqlite.Gnade</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL/Sqlite/Gnade</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql_fields.ads"</shell>
     <descr>GNATCOLL.SQL_Fields</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL__Fields</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-sql_impl.ads"</shell>
     <descr>GNATCOLL.SQL_Impl</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/SQL__Impl</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-scripts.ads"</shell>
     <descr>GNATCOLL.Scripts</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Scripts/&lt;Scripts&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-scripts-files.ads"</shell>
     <descr>GNATCOLL.Scripts.Files</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Scripts/Files</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-scripts-impl.ads"</shell>
     <descr>GNATCOLL.Scripts.Impl</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Scripts/Impl</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-scripts-projects.ads"</shell>
     <descr>GNATCOLL.Scripts.Projects</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Scripts/Projects</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-scripts-python.ads"</shell>
     <descr>GNATCOLL.Scripts.Python</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Scripts/Python</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-scripts-shell.ads"</shell>
     <descr>GNATCOLL.Scripts.Shell</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Scripts/Shell</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-scripts-utils.ads"</shell>
     <descr>GNATCOLL.Scripts.Utils</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Scripts/Utils</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-storage_pools.ads"</shell>
     <descr>GNATCOLL.Storage_Pools</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Storage__Pools/&lt;Storage__Pools&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-storage_pools-alignment.ads"</shell>
     <descr>GNATCOLL.Storage_Pools.Alignment</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Storage__Pools/Alignment</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-storage_pools-headers.ads"</shell>
     <descr>GNATCOLL.Storage_Pools.Headers</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Storage__Pools/Headers</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-strings.ads"</shell>
     <descr>GNATCOLL.Strings</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Strings</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-strings_impl.ads"</shell>
     <descr>GNATCOLL.Strings_Impl</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Strings__Impl</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-symbols.ads"</shell>
     <descr>GNATCOLL.Symbols</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Symbols</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-templates.ads"</shell>
     <descr>GNATCOLL.Templates</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Templates</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-terminal.ads"</shell>
     <descr>GNATCOLL.Terminal</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Terminal</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-traces.ads"</shell>
     <descr>GNATCOLL.Traces</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Traces/&lt;Traces&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-traces-syslog.ads"</shell>
     <descr>GNATCOLL.Traces.Syslog</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Traces/Syslog</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-tribooleans.ads"</shell>
     <descr>GNATCOLL.Tribooleans</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Tribooleans</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-utils.ads"</shell>
     <descr>GNATCOLL.Utils</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Utils</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-vfs.ads"</shell>
     <descr>GNATCOLL.VFS</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/VFS</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-vfs_types.ads"</shell>
     <descr>GNATCOLL.VFS_Types</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/VFS__Types</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-vfs_utils.ads"</shell>
     <descr>GNATCOLL.VFS_Utils</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/VFS__Utils</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-xref.ads"</shell>
     <descr>GNATCOLL.Xref</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Xref/&lt;Xref&gt;</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-xref-database.ads"</shell>
     <descr>GNATCOLL.Xref.Database</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Xref/Database</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

  <documentation_file>
     <shell>Editor.edit "gnatcoll-xref-database_names.ads"</shell>
     <descr>GNATCOLL.Xref.Database_Names</descr>
     <menu>/Help/GNAT Runtime/GNATCOLL/Xref/Database__Names</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

</GPS>'''
import GPS
GPS.parse_xml(XML)



from e3.testsuite.result import TestStatus
from drivers.data_validation import DataValidationDriver
import os
import json
import logging


class JSONValidationDriver(DataValidationDriver):

    def validate_result(self, process, data_file, result):
        # Read data file
        with open(os.path.join(self.test_env['test_dir'], data_file),
                  encoding='utf-8') as fd:
            expected = json.load(fd)

        got = json.loads(process.out.decode('utf-8'))
        if got != expected:
            # Escape non-ASCII codepoints. This is necessary to avoid errors
            # when logging wide characters on ancient encodings, such as CP1252
            # on Windows.
            got = got.encode('unicode_escape').decode('ascii')
            expected = expected.encode('unicode_escape').decode('ascii')
            logging.debug('%s\n<=>\n%s', got, expected)
            result.set_status(TestStatus.FAIL)
            result.push_result()
        return True
